﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using UnderGuardLibrary.DataTools;

namespace Capstone
{
    /* BUILD SHEETS TAB */
    //Author: Will Matheson
    partial class ManufacturingSystemForm
    {
        //LINQ to SQL
        private DataClassesDataContext dc;

        //our pretend "constructor"
        private void PopulateBuildSheetsTab() 
        {

            dc = new DataClassesDataContext();
            
            //get products
            _products = dc.getProducts().ToList();
            //get parts
            _parts = dc.getParts().ToList();
            
            
            //1. populate prod drop-down with productID, product Name
            foreach (Product product in _products)
            {
                buildSheetFindProductComboBox.Items.Add((string)(product.Product_ID + ". " + product.Product_Name));
            }

            //2. populate part drop-down with partID, part Name
            foreach (Part part in _parts)
            {
                buildSheetPartComboBox.Items.Add((string)(part.Part_ID + ". " + part.Part_Name));
            }
                    
        }

        private void buildSheetFindProductComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            //do nothing if this is being reset for some reason
            if (!(buildSheetFindProductComboBox.SelectedIndex >= 0))
            {
                return;
            }

            //clear part entry
            buildSheetDetailCancelButton_Click(sender, e);

            //enable parts selection
            buildSheetPartComboBox.Enabled = true;

            //get a copy of the currentProduct so it's easier to work with
            Product currentProduct = _products.ElementAt(buildSheetFindProductComboBox.SelectedIndex);

            //populate the status of the product
            buildSheetProductStatusVarLabel.Text = dc.getStatus(currentProduct.Status_Status_ID).ElementAt(0).Current_Status; 


            //get builds involving the selected product
            _builds = dc.getBuilds(currentProduct.Product_ID).ToList();
            //necessary after updates to quantities:
            dc.Refresh(System.Data.Linq.RefreshMode.OverwriteCurrentValues, _builds);

            //show the appropriate builds in the grid
            buildSheetsDataGridView.RowCount = 0;
            int row = 0;
            if (_builds.Count > 0)
            {
                buildSheetsDataGridView.Rows.Add(_builds.Count);

                foreach (Product_Part build in _builds)
                {
                    buildSheetsDataGridView.Rows[row].Cells[0].Value = build.Part_ID;
                    buildSheetsDataGridView.Rows[row].Cells[1].Value = (dc.getPart(build.Part_ID).ElementAt(0).Part_Name);
                    buildSheetsDataGridView.Rows[row].Cells[2].Value = build.Quantity;
                    row++;
                }
            }
        }

        private void buildSheetDetailInsertButton_Click(object sender, EventArgs e)
        {
            //check if quantity at least 1
            if (buildSheetQuantityMaskedTextBox.Text == String.Empty || !(Convert.ToInt16(buildSheetQuantityMaskedTextBox.Text) > 0))
            {
                MessageBox.Show("Quantity of new part must exceed 0.");
                return;
            }

            //check if part aleady exists on that product
            foreach (Product_Part build in _builds)
            {
                if (build.Part_ID == _parts.ElementAt(buildSheetPartComboBox.SelectedIndex).Part_ID)
                {
                    MessageBox.Show("Part already exists on product.");
                    return;
                }
            }

            //OK, do the insert:

            //make a new "build"
            Product_Part newBuild = new Product_Part();
            newBuild.Product_ID = _products.ElementAt(buildSheetFindProductComboBox.SelectedIndex).Product_ID;
            newBuild.Part_ID = _parts.ElementAt(buildSheetPartComboBox.SelectedIndex).Part_ID;
            newBuild.Quantity = Convert.ToInt16(buildSheetQuantityMaskedTextBox.Text);

            //put it in the database
            dc.insertBuild(newBuild.Product_ID, newBuild.Part_ID, newBuild.Quantity);
            
            //and refresh the view
            buildSheetFindProductComboBox_SelectedIndexChanged(sender, e);


        }

        private void buildSheetDetailEditSaveButton_Click(object sender, EventArgs e)
        {
            int buildIDToModify = -1;

            //check if quantity present
            if (buildSheetQuantityMaskedTextBox.Text == String.Empty || !(Convert.ToInt16(buildSheetQuantityMaskedTextBox.Text) >= 0))
            {
                MessageBox.Show("Quantity of part must be at least 0.");
                return;
            }

            //check if part doesn't aleady exist on that product
            bool partFound = false;
            foreach (Product_Part build in _builds)
            {
                if (build.Part_ID == _parts.ElementAt(buildSheetPartComboBox.SelectedIndex).Part_ID)
                {
                    partFound = true;
                    buildIDToModify = build.Product_Part_ID;
                }
            }
            if (!partFound)
            {
                MessageBox.Show("Part does not exist on product.");
                return;
            }

            //is quantity 0? that's the one to delete
            if (Convert.ToInt16(buildSheetQuantityMaskedTextBox.Text) == 0)
            {
                dc.deleteBuild(buildIDToModify);
                
            }
            else
            {
                //OK, do the update:

                //make a 'new' "build"
                Product_Part newBuild = new Product_Part();
                newBuild.Product_Part_ID = buildIDToModify;
                newBuild.Product_ID = _products.ElementAt(buildSheetFindProductComboBox.SelectedIndex).Product_ID;
                newBuild.Part_ID = _parts.ElementAt(buildSheetPartComboBox.SelectedIndex).Part_ID;
                newBuild.Quantity = Convert.ToInt16(buildSheetQuantityMaskedTextBox.Text);

                //put it in the database
                dc.updateBuild(newBuild.Product_Part_ID, newBuild.Product_ID, newBuild.Part_ID, newBuild.Quantity);
            }

            //and refresh the view
            buildSheetFindProductComboBox_SelectedIndexChanged(sender, e);
            
        }

        private void buildSheetDetailCancelButton_Click(object sender, EventArgs e)
        {
            buildSheetPartComboBox.SelectedIndex = -1;
            buildSheetDetailInsertButton.Enabled = false;
            buildSheetDetailEditSaveButton.Enabled = false;
            buildSheetDetailCancelButton.Enabled = false;
            buildSheetQuantityMaskedTextBox.Clear();
            buildSheetQuantityMaskedTextBox.Enabled = false;
            buildSheetPartStatusVarLabel.Text = String.Empty;
        }

        private void buildSheetPartComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!(buildSheetPartComboBox.SelectedIndex >= 0))
            {
                return;            
            }
            buildSheetDetailCancelButton.Enabled = true;
            buildSheetQuantityMaskedTextBox.Enabled = true;

            //populate the status of the part
            buildSheetPartStatusVarLabel.Text = dc.getStatus(_parts.ElementAt(buildSheetPartComboBox.SelectedIndex).Status_Status_ID).ElementAt(0).Current_Status;

            //enable either the insert OR the edit/save button:
            //check if part aleady exists on that product
            bool setUpdateMode = false;
            foreach (Product_Part build in _builds)
            {
                if (build.Part_ID == _parts.ElementAt(buildSheetPartComboBox.SelectedIndex).Part_ID)
                {
                    setUpdateMode = true;
                }
            }
            if (setUpdateMode)
            {
                buildSheetDetailEditSaveButton.Enabled = true;
                buildSheetDetailInsertButton.Enabled = false;
            }
            else 
            {
                buildSheetDetailInsertButton.Enabled = true;
                buildSheetDetailEditSaveButton.Enabled = false;
            }
            //fire this so if the qty box is populated, mode label will change
            buildSheetQuantityMaskedTextBox_TextChanged(sender, e);
        }

        private void buildSheetQuantityMaskedTextBox_TextChanged(object sender, EventArgs e)
        {
            if (buildSheetDetailEditSaveButton.Enabled == true)
            {
                if (buildSheetQuantityMaskedTextBox.Text == String.Empty)
                {
                    buildSheetDetailCrudModeLabel.Text = String.Empty;
                }
                else if (Convert.ToInt16(buildSheetQuantityMaskedTextBox.Text) > 0)
                {
                    buildSheetDetailCrudModeLabel.Text = "UPDATE";
                }
                else
                {
                    buildSheetDetailCrudModeLabel.Text = "DELETE";
                }
            }
            else if (buildSheetDetailInsertButton.Enabled == true)
            {
                if (buildSheetQuantityMaskedTextBox.Text == String.Empty)
                {
                    buildSheetDetailCrudModeLabel.Text = String.Empty;
                }
                else if (Convert.ToInt16(buildSheetQuantityMaskedTextBox.Text) > 0)
                {
                    buildSheetDetailCrudModeLabel.Text = "INSERT";
                }
                else
                {
                    buildSheetDetailCrudModeLabel.Text = String.Empty;
                }
            }
            else //if clearing the box for some other reason
            {
                buildSheetDetailCrudModeLabel.Text = String.Empty;            
            }
        }

        private void buildSheetsDataGridView_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (!(buildSheetsDataGridView.RowCount > 0))
            {
                return;            
            }

            //get the partID from the selected row
            int partID = (int)buildSheetsDataGridView.CurrentRow.Cells[0].Value;
            int quantity = (int)buildSheetsDataGridView.CurrentRow.Cells[2].Value;

            //select appropriate part
            for (int i = 0; i < _parts.Count; i++)
            {
                if (partID == _parts.ElementAt(i).Part_ID)
                {
                    buildSheetPartComboBox.SelectedIndex = i;                
                }
            }

            //set quantity
            buildSheetQuantityMaskedTextBox.Text = quantity.ToString();
        }
    }

    
}
