﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.Windows.Forms;

namespace Capstone
{
    /* ORDERS TAB */
    //Author: Jamal Rahhali
    partial class ManufacturingSystemForm
    {
        private DataSet dataSet = new DataSet();
        DataRow editingRow;
        SqlDataAdapter ordersDataAdapter;
        BindingSource clientBindingSource;
        BindingSource orderBindingSource;
        private static bool ordersTabInitialized = false;

        private void InitializeOrdersTab()
        {
            if (ordersTabInitialized)
                return;
            else
                ordersTabInitialized = true;

            SqlDataAdapter clientDataAdapter = new SqlDataAdapter("SELECT * FROM Clients", CapstoneGlobals.UnderGuardConnection);
            ordersDataAdapter = new SqlDataAdapter("SELECT * FROM Orders INNER JOIN Status ON (Orders.Status_Status_ID = Status.Status_ID)", CapstoneGlobals.UnderGuardConnection);
            ordersDataAdapter.UpdateCommand = CapstoneGlobals.UnderGuardConnection.CreateCommand();
            ordersDataAdapter.UpdateCommand.CommandText =
                "UPDATE Orders SET Status_Status_ID = @Status_Status_ID WHERE Order_ID = @Order_ID";
            ordersDataAdapter.UpdateCommand.Parameters.Add("@Status_Status_ID", SqlDbType.Int, sizeof(Int32), "Status_Status_ID");
            ordersDataAdapter.UpdateCommand.Parameters.Add("@Order_ID", SqlDbType.Int, sizeof(Int32), "Order_ID");
            SqlDataAdapter statusDataAdapter = new SqlDataAdapter("SELECT * FROM Status", CapstoneGlobals.UnderGuardConnection);

            clientDataAdapter.Fill(dataSet, "Client");
            ordersDataAdapter.Fill(dataSet, "Order");
            statusDataAdapter.Fill(dataSet, "Status");
            DataRelation relation = new DataRelation("ClientOrderRelation",
                dataSet.Tables["Client"].Columns["Client_ID"],
                dataSet.Tables["Order"].Columns["Client_ID"]);
            dataSet.Relations.Add(relation);

            clientBindingSource = new BindingSource();
            clientBindingSource.DataSource = dataSet;
            clientBindingSource.DataMember = "Client";

            orderBindingSource = new BindingSource();
            orderBindingSource.DataSource = clientBindingSource;
            orderBindingSource.DataMember = "ClientOrderRelation";

            orderFindClientComboBox.DataSource = clientBindingSource;
            orderFindClientComboBox.DisplayMember = "Client_Name";

            ordersDataGridView.AutoGenerateColumns = false;
            ordersDataGridView.DataSource = orderBindingSource;
            ordersDataGridView.Columns[0].DataPropertyName = "Order_ID";
            ordersDataGridView.Columns[1].DataPropertyName = "Current_Status";
            ordersDataGridView.Columns[2].DataPropertyName = "Date";

            orderStatusComboBox.DataSource = dataSet.Tables["Status"];
            orderStatusComboBox.DisplayMember = "Current_Status";
            orderStatusComboBox.ValueMember = "Status_ID";
            orderStatusComboBox.DataBindings.Add("SelectedValue", clientBindingSource, "ClientOrderRelation.Status_Status_ID");

            orderEditSaveButton.Text = "Edit";
            orderCrudModeLabel.Text = "Viewing";
            orderGroupBox.Enabled = false;
            orderCancelButton.Enabled = false;
        }

        private void orderFindClientComboBox_SelectedValueChanged(object sender, EventArgs e)
        {

        }

        private void orderEditSaveButton_Click(object sender, EventArgs e)
        {
            orderFindClientComboBox.Enabled = false;
            ordersDataGridView.Enabled = false;
            if (orderCrudModeLabel.Text == "Viewing")
            {
                orderEditSaveButton.Text = "Save";
                orderCrudModeLabel.Text = "Editing...";
                orderGroupBox.Enabled = true;
                orderCancelButton.Enabled = true;

                editingRow = ((DataRowView)ordersDataGridView.CurrentRow.DataBoundItem).Row;
                editingRow.BeginEdit();
            }
            else //saving
            {
                orderEditSaveButton.Text = "Edit";
                orderCrudModeLabel.Text = "Viewing";
                orderGroupBox.Enabled = false;
                orderCancelButton.Enabled = false;
                editingRow["Current_Status"] = orderStatusComboBox.Text;
                editingRow.EndEdit();
                DataSet changedData = dataSet.GetChanges(DataRowState.Modified);
                ordersDataAdapter.Update(changedData, "Order");
                dataSet.AcceptChanges();
                orderFindClientComboBox.Enabled = true;
                ordersDataGridView.Enabled = true;
            }
        }

        private void orderCancelButton_Click(object sender, EventArgs e)
        {
            orderEditSaveButton.Text = "Edit";
            orderCrudModeLabel.Text = "Viewing";
            orderGroupBox.Enabled = false;
            orderCancelButton.Enabled = false;
            orderFindClientComboBox.Enabled = true;
            ordersDataGridView.Enabled = true;

            editingRow.CancelEdit();
        }

        private void orderFindClientComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {

        }
    }
}