﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.Windows.Forms;
using System.Drawing;
using System.ComponentModel;

namespace Capstone
{
    /* PRODUCTS TAB */
    //Author: Jamal Rahhali
    partial class ManufacturingSystemForm
    {
        private DataTable productsTable;
        private SqlDataAdapter dataAdapter;
        private BindingList<StatusP> statusList = new BindingList<StatusP>();
        private BindingList<CategoryP> discludedCategoryList = new BindingList<CategoryP>();
        private BindingList<CategoryP> includedCategoryList = new BindingList<CategoryP>();
        private List<CategoryP> originalCategoryList;
        private BindingList<CatalogP> discludedCatalogList = new BindingList<CatalogP>();
        private BindingList<CatalogP> includedCatalogList = new BindingList<CatalogP>();
        private List<CatalogP> originalCatalogList;
        private int lastProductSelected = -1;

        public class StatusP
        {
            public StatusP(int statusId, string statusName)
            {
                StatusId = statusId;
                StatusName = statusName;
            }

            public int StatusId { get; set; }
            public string StatusName { get; set; }

            public override string ToString() { return StatusName; }
        }

        public class CategoryP
        {
            public CategoryP(int categoryId, string categoryName)
            {
                CategoryId = categoryId;
                CategoryName = categoryName;

            }
            public int CategoryId { get; set; }
            public string CategoryName { get; set; }

            public override string ToString() { return CategoryName; }
        }

        public class CatalogP
        {
            public CatalogP(int catalogId, string catalogName)
            {
                CatalogId = catalogId;
                CatalogName = catalogName;

            }
            public int CatalogId { get; set; }
            public string CatalogName { get; set; }

            public override string ToString() { return CatalogName; }
        }

        public enum FillMode
        {
            CurrentProduct,
            NewProduct
        }

        private void InitializeProductsTab()
        {
            FillProductDataGridView();
            FillProductStatusComboBox();
            UpdateProductFields();
            productGroupBox.Enabled = false;
            productCrudModeLabel.Text = "Viewing";
            productEditSaveButton.Text = "Edit";
        }

        private void productsDataGridView_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (GetProductDataAtColumn<int>("Product_ID") != lastProductSelected)
            {
                UpdateProductFields();
            }
        }

        private void productImageBrowseButton_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            DialogResult result = dialog.ShowDialog();
            if (result == DialogResult.OK)
            {
                productImageTextBox.Text = dialog.FileName;
            }
        }

        private void productInsertButton_Click(object sender, EventArgs e)
        {
            productInsertButton.Enabled = false;
            productEditSaveButton.Text = "Save";
            productGroupBox.Enabled = true;
            productsDataGridView.Enabled = false;
            productCrudModeLabel.Text = "Inserting...";
            ClearProductFields();
            originalCatalogList = new List<CatalogP>();
            originalCategoryList = new List<CategoryP>();
        }

        private void productCancelButton_Click(object sender, EventArgs e)
        {
            productInsertButton.Enabled = true;
            productEditSaveButton.Text = "Edit";
            productGroupBox.Enabled = false;
            UpdateProductFields();
            productsDataGridView.Enabled = true;
            productCrudModeLabel.Text = "Viewing";
        }

        private void productEditSaveButton_Click(object sender, EventArgs e)
        {
            if (productEditSaveButton.Text == "Edit")
            {
                productGroupBox.Enabled = true;
                productEditSaveButton.Text = "Save";
                productInsertButton.Enabled = false;
                productsDataGridView.Enabled = false;
                productCrudModeLabel.Text = "Editing...";
                originalCatalogList = new List<CatalogP>(includedCatalogList);
                originalCategoryList = new List<CategoryP>(includedCategoryList);
            }
            else // save
            {
                SaveProductDetails();
            }
        }

        private void productIncludeInCatelogButton_Click(object sender, EventArgs e)
        {
            includedCatalogList.Add(discludedCatalogList.ElementAt(productNotIncludedInCatalogListBox.SelectedIndex));
            discludedCatalogList.RemoveAt(productNotIncludedInCatalogListBox.SelectedIndex);
            SetCatalogIncludeRemoveButtonState();
        }

        private void productRemoveFromCatelogButton_Click(object sender, EventArgs e)
        {
            discludedCatalogList.Add(includedCatalogList.ElementAt(productIncludedInCatalogListBox.SelectedIndex));
            includedCatalogList.RemoveAt(productIncludedInCatalogListBox.SelectedIndex);
            SetCatalogIncludeRemoveButtonState();
        }

        private void productIncludedInCategoryButton_Click(object sender, EventArgs e)
        {
            includedCategoryList.Add(discludedCategoryList.ElementAt(productCategoriesListBox.SelectedIndex));
            discludedCategoryList.RemoveAt(productCategoriesListBox.SelectedIndex);
            SetCategoryIncludeRemoveButtonState();
        }

        private void productRemoveFromCategoryButton_Click(object sender, EventArgs e)
        {
            discludedCategoryList.Add(includedCategoryList.ElementAt(productIncludedInCategoriesListBox.SelectedIndex));
            includedCategoryList.RemoveAt(productIncludedInCategoriesListBox.SelectedIndex);
            SetCategoryIncludeRemoveButtonState();
        }

        private void SaveProductDetails()
        {
            int id = 0;
            SqlCommand command;
            if (productCrudModeLabel.Text == "Editing...")
            {
                command = new SqlCommand("ProductsTab_UpdateProduct", CapstoneGlobals.UnderGuardConnection);
                command.CommandType = CommandType.StoredProcedure;
                id = GetProductDataAtColumn<int>("Product_ID");
                command.Parameters.AddWithValue("@ProductID", id);
                command.Parameters.AddWithValue("@ProductName", productNameTextBox.Text);
                command.Parameters.AddWithValue("@ProductDescription", productDescriptionTextBox.Text);
                command.Parameters.AddWithValue("@ProductMfgTime", productMfgTimeTextBox.Text);
                command.Parameters.AddWithValue("@ProductImagePath", productImageTextBox.Text);
                command.Parameters.AddWithValue("@ProductPrice", Convert.ToDouble(productPriceTextBox.Text.Replace("$", "")));
                command.Parameters.AddWithValue("@ProductStatusID", ((StatusP)productStatusComboBox.SelectedItem).StatusId);
                CapstoneGlobals.UnderGuardConnection.Open();
                command.ExecuteNonQuery();
            }
            else // saving
            {
                command = new SqlCommand("ProductsTab_InsertProduct", CapstoneGlobals.UnderGuardConnection);
                command.CommandType = CommandType.StoredProcedure;
                command.Parameters.AddWithValue("@ProductName", productNameTextBox.Text);
                command.Parameters.AddWithValue("@ProductDescription", productDescriptionTextBox.Text);
                command.Parameters.AddWithValue("@ProductMfgTime", productMfgTimeTextBox.Text);
                command.Parameters.AddWithValue("@ProductImagePath", productImageTextBox.Text);
                command.Parameters.AddWithValue("@ProductPrice", Convert.ToDouble(productPriceTextBox.Text.Replace("$", "")));
                command.Parameters.AddWithValue("@ProductStatusID", ((StatusP)productStatusComboBox.SelectedItem).StatusId);
                CapstoneGlobals.UnderGuardConnection.Open();
                command.ExecuteNonQuery();
                SqlCommand command2 = new SqlCommand("SELECT MAX(Product_ID) FROM Products", CapstoneGlobals.UnderGuardConnection);
                id = (int)command2.ExecuteScalar();
            }

            // CatalogP
            for (int i = 0; i < includedCatalogList.Count; i++)
            {
                if (!originalCatalogList.Contains(includedCatalogList[i]))
                {
                    //insert it
                    command = new SqlCommand("ProductsTab_InsertProductCatalog", CapstoneGlobals.UnderGuardConnection);
                    command.CommandType = CommandType.StoredProcedure;
                    command.Parameters.AddWithValue("@ProductID", id);
                    command.Parameters.AddWithValue("@CatalogID", ((CatalogP)includedCatalogList[i]).CatalogId);
                    command.ExecuteNonQuery();
                }
            }

            for (int i = 0; i < originalCatalogList.Count; i++)
            {
                if (!includedCatalogList.Contains(originalCatalogList[i]))
                {
                    //delete it
                    command = new SqlCommand("ProductsTab_DeleteProductCatalog", CapstoneGlobals.UnderGuardConnection);
                    command.CommandType = CommandType.StoredProcedure;
                    command.Parameters.AddWithValue("@ProductID", id);
                    command.Parameters.AddWithValue("@CatalogID", ((CatalogP)originalCatalogList[i]).CatalogId);
                    command.ExecuteNonQuery();
                }
            }

            // CategoryP
            for (int i = 0; i < includedCategoryList.Count; i++)
            {
                if (!originalCategoryList.Contains(includedCategoryList[i]))
                {
                    //insert it
                    command = new SqlCommand("ProductsTab_InsertProductCategory", CapstoneGlobals.UnderGuardConnection);
                    command.CommandType = CommandType.StoredProcedure;
                    command.Parameters.AddWithValue("@ProductID", id);
                    command.Parameters.AddWithValue("@CategoryID", ((CategoryP)includedCategoryList[i]).CategoryId);
                    command.ExecuteNonQuery();
                }
            }

            for (int i = 0; i < originalCategoryList.Count; i++)
            {
                if (!includedCategoryList.Contains(originalCategoryList[i]))
                {
                    //delete it
                    command = new SqlCommand("ProductsTab_DeleteProductCategory", CapstoneGlobals.UnderGuardConnection);
                    command.CommandType = CommandType.StoredProcedure;
                    command.Parameters.AddWithValue("@ProductID", id);
                    command.Parameters.AddWithValue("@CategoryID", ((CategoryP)originalCategoryList[i]).CategoryId);
                    command.ExecuteNonQuery();
                }
            }

            command.Dispose();
            CapstoneGlobals.UnderGuardConnection.Close();
            FillProductDataGridView();

            productInsertButton.Enabled = true;
            productEditSaveButton.Text = "Edit";
            productGroupBox.Enabled = false;
            UpdateProductFields();
            productsDataGridView.Enabled = true;
            productCrudModeLabel.Text = "Viewing";
        }

        private void FillProductDataGridView()
        {
            // Prevent all the columns in the DataSet from showing
            productsDataGridView.AutoGenerateColumns = false;

            SqlCommand command = new SqlCommand("ProductsTab_GetAllProducts", CapstoneGlobals.UnderGuardConnection);
            command.CommandType = CommandType.StoredProcedure;
            dataAdapter = new SqlDataAdapter(command);
            productsTable = new DataTable("Products");
            dataAdapter.Fill(productsTable);

            // Data bind the DataGridView
            productsDataGridView.DataSource = productsTable;
            productsDataGridView.Columns["productIDColumn"].DataPropertyName = "Product_ID";
            productsDataGridView.Columns["productStatusColumn"].DataPropertyName = "Current_Status";
            productsDataGridView.Columns["productPriceColumn"].DefaultCellStyle.Format = "C";
            productsDataGridView.Columns["productPriceColumn"].DataPropertyName = "Price";
            productsDataGridView.Columns["productNameColumn"].DataPropertyName = "Product_Name";
        }

        private void FillProductStatusComboBox()
        {
            SqlCommand command = new SqlCommand("ProductsTab_GetAllStatuses", CapstoneGlobals.UnderGuardConnection);
            command.CommandType = CommandType.StoredProcedure;
            CapstoneGlobals.UnderGuardConnection.Open();
            SqlDataReader reader = command.ExecuteReader();
            statusList.Clear(); // new BindingList<StatusP>();
            while (reader.Read())
            {
                statusList.Add(new StatusP((int)reader["Status_ID"], (string)reader["Current_Status"]));
            }
            reader.Close();
            CapstoneGlobals.UnderGuardConnection.Close();

            productStatusComboBox.DataSource = statusList;
            productStatusComboBox.DisplayMember = "StatusName";
            productStatusComboBox.ValueMember = "StatusName";
        }

        private void UpdateProductFields()
        {
            productStatusComboBox.SelectedValue = GetProductDataAtColumn<string>("Current_Status");
            productNameTextBox.Text = GetProductDataAtColumn<string>("Product_Name");
            productPriceTextBox.Text = String.Format("{0:C}", GetProductDataAtColumn<double>("Price"));
            productImageTextBox.Text = GetProductDataAtColumn<string>("Image_Path");
            productDescriptionTextBox.Text = GetProductDataAtColumn<string>("Product_Description");
            productMfgTimeTextBox.Text = (GetProductDataAtColumn<int>("Manufacturing_Time")).ToString();
            FillIncludedProductCatalogs();
            FillDiscludedProductCatalogs();
            FillIncludedProductCategories();
            FillDiscludedProductCategories();
            SetCatalogIncludeRemoveButtonState();
            SetCategoryIncludeRemoveButtonState();
        }

        private void SetCatalogIncludeRemoveButtonState()
        {
            if (productIncludedInCatalogListBox.Items.Count == 0)
            {
                productRemoveFromCatelogButton.Enabled = false;
            }
            else
            {
                productRemoveFromCatelogButton.Enabled = true;
            }

            if (productNotIncludedInCatalogListBox.Items.Count == 0)
            {
                productIncludeInCatelogButton.Enabled = false;
            }
            else
            {
                productIncludeInCatelogButton.Enabled = true;
            }
        }

        private void SetCategoryIncludeRemoveButtonState()
        {
            if (productIncludedInCategoriesListBox.Items.Count == 0)
            {
                productRemoveFromCategoryButton.Enabled = false;
            }
            else
            {
                productRemoveFromCategoryButton.Enabled = true;
            }

            if (productCategoriesListBox.Items.Count == 0)
            {
                productIncludedInCategoryButton.Enabled = false;
            }
            else
            {
                productIncludedInCategoryButton.Enabled = true;
            }
        }

        private void ClearProductFields()
        {
            productStatusComboBox.SelectedIndex = 0;
            productNameTextBox.Text = "";
            productPriceTextBox.Text = "";
            productImageTextBox.Text = "";
            productDescriptionTextBox.Text = "";
            productMfgTimeTextBox.Text = "";
            includedCategoryList.Clear();
            includedCatalogList.Clear();
            FillDiscludedProductCatalogs(FillMode.NewProduct);
            FillDiscludedProductCategories(FillMode.NewProduct);
            SetCatalogIncludeRemoveButtonState();
            SetCategoryIncludeRemoveButtonState();
        }

        private T GetProductDataAtColumn<T>(string columnName)
        {

                DataRowView dataRowView = (DataRowView)productsDataGridView.BindingContext[productsTable].Current;
                return dataRowView.Row.Field<T>(columnName);
        }

        private void FillIncludedProductCatalogs()
        {
            // Set up the stored procedure
            SqlCommand command = new SqlCommand("ProductsTab_GetIncludedCatalogsForProduct", CapstoneGlobals.UnderGuardConnection);
            command.CommandType = CommandType.StoredProcedure;
            command.Parameters.AddWithValue("@ProductID", GetProductDataAtColumn<int>("Product_ID"));

            // Execute the stored procedure
            CapstoneGlobals.UnderGuardConnection.Open();
            SqlDataReader reader = command.ExecuteReader();
            includedCatalogList.Clear(); // = new BindingList<CatalogP>();
            while (reader.Read())
            {
                includedCatalogList.Add(new CatalogP((int)reader["Catalog_ID"], (string)reader["Catalog_Name"]));
            }
            reader.Close();
            CapstoneGlobals.UnderGuardConnection.Close();

            // Data bind the result
            productIncludedInCatalogListBox.DataSource = includedCatalogList;
            productIncludedInCatalogListBox.DisplayMember = "CatalogName";
            productIncludedInCatalogListBox.ValueMember = "CatalogId";
        }

        private void FillDiscludedProductCatalogs(FillMode mode = FillMode.CurrentProduct)
        {
            // Set up the stored procedure
            SqlCommand command = new SqlCommand("ProductsTab_GetDiscludedCatalogsForProduct", CapstoneGlobals.UnderGuardConnection);
            command.CommandType = CommandType.StoredProcedure;
            int productId = (mode == FillMode.CurrentProduct) ? GetProductDataAtColumn<int>("Product_ID") : -1;
            command.Parameters.AddWithValue("@ProductID", productId);

            // Execute the stored procedure
            CapstoneGlobals.UnderGuardConnection.Open();
            SqlDataReader reader = command.ExecuteReader();
            discludedCatalogList.Clear(); // = new BindingList<CatalogP>();
            while (reader.Read())
            {
                discludedCatalogList.Add(new CatalogP((int)reader["Catalog_ID"], (string)reader["Catalog_Name"]));
            }
            reader.Close();
            CapstoneGlobals.UnderGuardConnection.Close();

            // Data bind the result
            productNotIncludedInCatalogListBox.DataSource = discludedCatalogList;
            productNotIncludedInCatalogListBox.DisplayMember = "CatalogName";
            productNotIncludedInCatalogListBox.ValueMember = "CatalogId";
        }

        private void FillIncludedProductCategories()
        {
            // Set up the stored procedure
            SqlCommand command = new SqlCommand("ProductsTab_GetIncludedCategoriesForProduct", CapstoneGlobals.UnderGuardConnection);
            command.CommandType = CommandType.StoredProcedure;
            command.Parameters.AddWithValue("@ProductID", GetProductDataAtColumn<int>("Product_ID"));

            // Execute the stored procedure
            CapstoneGlobals.UnderGuardConnection.Open();
            SqlDataReader reader = command.ExecuteReader();
            includedCategoryList.Clear(); // = new BindingList<CategoryP>();
            while (reader.Read())
            {
                includedCategoryList.Add(new CategoryP((int)reader["Category_ID"], (string)reader["Category_Name"]));
            }
            reader.Close();
            CapstoneGlobals.UnderGuardConnection.Close();

            // Data bind the result
            productIncludedInCategoriesListBox.DataSource = includedCategoryList;
            productIncludedInCategoriesListBox.DisplayMember = "CategoryName";
            productIncludedInCategoriesListBox.ValueMember = "CategoryId";
        }

        private void FillDiscludedProductCategories(FillMode mode = FillMode.CurrentProduct)
        {
            // Set up the stored procedure
            SqlCommand command = new SqlCommand("ProductsTab_GetDiscludedCategoriesForProduct", CapstoneGlobals.UnderGuardConnection);
            command.CommandType = CommandType.StoredProcedure;
            int productId = (mode == FillMode.CurrentProduct) ? GetProductDataAtColumn<int>("Product_ID") : -1;
            command.Parameters.AddWithValue("@ProductID", productId);

            // Execute the stored procedure
            CapstoneGlobals.UnderGuardConnection.Open();
            SqlDataReader reader = command.ExecuteReader();
            discludedCategoryList.Clear(); // = new BindingList<CategoryP>();
            while (reader.Read())
            {
                discludedCategoryList.Add(new CategoryP((int)reader["Category_ID"], (string)reader["Category_Name"]));
            }
            reader.Close();
            CapstoneGlobals.UnderGuardConnection.Close();

            // Data bind the result
            productCategoriesListBox.Text = "";
            productCategoriesListBox.DataSource = discludedCategoryList;
            productCategoriesListBox.DisplayMember = "CategoryName";
            productCategoriesListBox.ValueMember = "CategoryId";
        }
    }
}