﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnderGuardLibrary.DataTools;
using System.Windows.Forms;

namespace Capstone
{
    /* CLIENTS TAB */
    //Author: Will Matheson
    partial class OrderingSystemForm
    {
        //pk storage from datagrid (in case the selection is moved w/o clicking)
        int clientPK;

        //our pretend "constructor"
        private void PopulateClientsTab()
        {
            //fill in provinces
            fillProvinces();

            //get clients, show in datagrid
            RefreshClientData();

        }

        private void fillProvinces()
        {
            //should really be a provinces table in the db
            List<string> cdnSubNat = new List<string>();
            cdnSubNat.Add("AB");
            cdnSubNat.Add("BC");
            cdnSubNat.Add("MB");
            cdnSubNat.Add("NB");
            cdnSubNat.Add("NL");
            cdnSubNat.Add("NS");
            cdnSubNat.Add("NT");
            cdnSubNat.Add("NU");
            cdnSubNat.Add("ON");
            cdnSubNat.Add("PE");
            cdnSubNat.Add("QC");
            cdnSubNat.Add("SK");
            cdnSubNat.Add("YT");
            List<string> listCopy = new List<string>(cdnSubNat);
            List<string> anotherCopy = new List<string>(cdnSubNat);
            clientBillingProvinceComboBox.DataSource = cdnSubNat;
            clientShippingProvinceComboBox.DataSource = listCopy;
            //contacts tab
            contactProvinceComboBox.DataSource = anotherCopy;
        }

        private void RefreshClientData()
        {
            //_clients.Clear();
            _clients = dc.getClients().ToList();
            // necessary after updates to clients:
            dc.Refresh(System.Data.Linq.RefreshMode.OverwriteCurrentValues, _clients);

            //fill grid
            clientDataGridView.RowCount = 0;
            int row = 0;
            if (_clients.Count > 0)
            {
                clientDataGridView.Rows.Add(_clients.Count);

                foreach (Client client in _clients)
                {
                    clientDataGridView.Rows[row].Cells[0].Value = client.Client_ID;
                    clientDataGridView.Rows[row].Cells[1].Value = client.Client_Name;
                    row++;
                }
            }
       
        }

        private void clientDataGridView_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (!(clientDataGridView.RowCount > 0))
            {
                return;
            }

            //save the pk
            clientPK = (int)clientDataGridView.CurrentRow.Cells[0].Value;

            clientCrudModeLabel.Text = "UPDATE";
            clientInsertButton.Text = "Delete";
            clientEditSaveButton.Enabled = true;

            //get the client
            Client currentClient = dc.getClient((int)clientDataGridView.CurrentRow.Cells[0].Value).Single();
            clientNameTextBox.Text = currentClient.Client_Name;

            //get billing address
            Address billingAddress = dc.getAddress(currentClient.Billing_Address).Single();

            //populate billing address
            clientBillingAddressTextBox.Text = billingAddress.Civic_Address;
            clientBillingCityTextBox.Text = billingAddress.City;
            clientBillingProvinceComboBox.SelectedItem = billingAddress.Province;
            clientBillingPostalCodeTextBox.Text = billingAddress.Postal_Code;
            
            //get shipping address if necessary
            if (currentClient.Billing_Address != currentClient.Shipping_Address)
            {
                clientShippingSameCheckBox.Checked = false;
                Address shippingAddress = dc.getAddress(currentClient.Shipping_Address).Single();

                //populate shipping address
                clientShippingAddressTextBox.Text = shippingAddress.Civic_Address;
                clientShippingCityTextBox.Text = shippingAddress.City;
                clientShippingProvinceComboBox.SelectedItem = shippingAddress.Province;
                clientShippingPostalCodeTextBox.Text = shippingAddress.Postal_Code;
            }
            else 
            {
                clientShippingSameCheckBox.Checked = true;
                clientShippingAddressTextBox.Clear();
                clientShippingPostalCodeTextBox.Clear();
                clientShippingCityTextBox.Clear();
                clientShippingProvinceComboBox.SelectedIndex = 0;
            }
        }

        private void clientShippingSameCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (clientShippingSameCheckBox.Checked)
            {
                clientShippingAddressGroupBox.Enabled = false;
            }
            else
            {
                clientShippingAddressGroupBox.Enabled = true;            
            }

        }

        #pragma warning disable
        private void clientEditSaveButton_Click(object sender, EventArgs e)
        {
            if (clientFieldCheck() && clientPK != null)
            {
                Client updatedClient = new Client();
                updatedClient.Client_Name = clientNameTextBox.Text;

                //get the PK for the client
                updatedClient.Client_ID = clientPK;
                //get the address IDs
                Client oldClient = dc.getClient(updatedClient.Client_ID).Single();
                updatedClient.Billing_Address = oldClient.Billing_Address;
                updatedClient.Shipping_Address = oldClient.Shipping_Address;
                
                Address newBillingAddress = new Address();
                Address newShippingAddress = new Address();
                newBillingAddress.Civic_Address = clientBillingAddressTextBox.Text;
                newBillingAddress.City = clientBillingCityTextBox.Text;
                newBillingAddress.Province = clientBillingProvinceComboBox.Text;
                newBillingAddress.Postal_Code = clientBillingPostalCodeTextBox.Text;

                //change the billing address
                dc.updateAddress(updatedClient.Billing_Address, newBillingAddress.Civic_Address, newBillingAddress.City, newBillingAddress.Province, newBillingAddress.Postal_Code, "UpdatedClient-Billing");

                if (!clientShippingSameCheckBox.Checked)
                {
                    newShippingAddress.Civic_Address = clientShippingAddressTextBox.Text;
                    newShippingAddress.City = clientShippingCityTextBox.Text;
                    newShippingAddress.Province = clientShippingProvinceComboBox.Text;
                    newShippingAddress.Postal_Code = clientShippingPostalCodeTextBox.Text;

                    //insert new shipping address (didn't exist before)
                    if (oldClient.Billing_Address == oldClient.Shipping_Address)
                    {
                        var insertShipAddressResult = dc.insertAddress(newShippingAddress.Civic_Address, newShippingAddress.City, newShippingAddress.Province, newShippingAddress.Postal_Code, "UpdatedClient-Shipping");
                        updatedClient.Shipping_Address = (int)insertShipAddressResult.ElementAt(0).ID;
                    }
                    //or change the shipping address
                    else                    
                    {
                        dc.updateAddress(updatedClient.Shipping_Address, newShippingAddress.Civic_Address, newShippingAddress.City, newShippingAddress.Province, newShippingAddress.Postal_Code, "UpdatedClient-Shipping");                    
                    }
                }
                //we may orphan an old shipping address, but the cure is worse than the disease
                else 
                {
                    updatedClient.Shipping_Address = updatedClient.Billing_Address;
                }

                //update the client
                dc.updateClient(updatedClient.Client_ID, updatedClient.Client_Name, updatedClient.Billing_Address, updatedClient.Shipping_Address);

                //refresh
                RefreshClientData();

                clientCancelButton_Click(sender, e);
            }

        }

        private void clientCancelButton_Click(object sender, EventArgs e)
        {
            clientNameTextBox.Clear();
            clientBillingAddressTextBox.Clear();
            clientBillingCityTextBox.Clear();
            clientBillingPostalCodeTextBox.Clear();
            clientBillingProvinceComboBox.SelectedIndex = 0;
            clientShippingSameCheckBox.Checked = false;
            clientShippingAddressTextBox.Clear();
            clientShippingCityTextBox.Clear();
            clientShippingPostalCodeTextBox.Clear();
            clientShippingProvinceComboBox.SelectedIndex = 0;
            clientDataGridView.ClearSelection();
            clientInsertButton.Text = "Insert";
            clientCrudModeLabel.Text = "INSERT";
            clientEditSaveButton.Enabled = false;
        }

        private bool clientFieldCheck()
        {
            if (clientNameTextBox.Text == string.Empty)
            {
                MessageBox.Show("Client must have a name.");
                return false;            
            }

            if (clientBillingAddressTextBox.Text == string.Empty)
            {
                MessageBox.Show("Billing address field Address required.");
                return false;            
            }

            if (clientBillingCityTextBox.Text == string.Empty)
            {
                MessageBox.Show("Billing address field City required.");
                return false;            
            }

            if (clientBillingPostalCodeTextBox.Text == string.Empty)
            {
                MessageBox.Show("Billing address field Postal Code required.");
                return false;            
            }

            if (!clientShippingSameCheckBox.Checked)
            {
                if (clientShippingAddressTextBox.Text == string.Empty)
                {
                    MessageBox.Show("Shipping address field Address required.");
                    return false;            
                }

                if (clientShippingCityTextBox.Text == string.Empty)
                {
                    MessageBox.Show("Shipping address field City required.");
                    return false;            
                }

                if (clientShippingPostalCodeTextBox.Text == string.Empty)
                {
                    MessageBox.Show("Shipping address field Postal Code required.");
                    return false;            
                }            
            }

            return true;        
        }

        private void deleteClient(object sender, EventArgs e)
        {
            clientCrudModeLabel.Text = "DELETE";

            //http://stackoverflow.com/a/3036880
            DialogResult dialogResult = MessageBox.Show("Are you sure you want to delete this client?", "Confirm Delete", MessageBoxButtons.YesNo);
            if (dialogResult == DialogResult.Yes && clientPK != null)
            {
                dc.deleteClient(clientPK);
                RefreshClientData();
                clientCancelButton_Click(sender, e);
            }
            else if (dialogResult == DialogResult.No)
            {
                //MessageBox.Show("Ok, you're safe.");
            }            
        }

        private void clientInsertButton_Click(object sender, EventArgs e)
        {
            if  (clientInsertButton.Text == "Delete")
            {
                deleteClient(sender, e);
                return;            
            }
            if (clientFieldCheck())
            {
                Address newBillingAddress = new Address();
                Address newShippingAddress = new Address();
                newBillingAddress.Civic_Address = clientBillingAddressTextBox.Text;
                newBillingAddress.City = clientBillingCityTextBox.Text;
                newBillingAddress.Province = clientBillingProvinceComboBox.Text;
                newBillingAddress.Postal_Code = clientBillingPostalCodeTextBox.Text;

                //insert the address
                var insertBillAddressResult = dc.insertAddress(newBillingAddress.Civic_Address, newBillingAddress.City, newBillingAddress.Province, newBillingAddress.Postal_Code, "NewClient-Billing");
                newBillingAddress.Address_ID = (int)insertBillAddressResult.ElementAt(0).ID;
                newShippingAddress.Address_ID = newBillingAddress.Address_ID;
                
                if (!clientShippingSameCheckBox.Checked)
                {
                    newShippingAddress.Civic_Address = clientShippingAddressTextBox.Text;
                    newShippingAddress.City = clientShippingCityTextBox.Text;
                    newShippingAddress.Province = clientShippingProvinceComboBox.Text;
                    newShippingAddress.Postal_Code = clientShippingPostalCodeTextBox.Text;

                    //insert the address
                    var insertShipAddressResult = dc.insertAddress(newShippingAddress.Civic_Address, newShippingAddress.City, newShippingAddress.Province, newShippingAddress.Postal_Code, "NewClient-Shipping");
                    newShippingAddress.Address_ID = (int)insertShipAddressResult.ElementAt(0).ID;
                }
                
                Client newClient = new Client();
                newClient.Billing_Address = newBillingAddress.Address_ID;
                newClient.Shipping_Address = newShippingAddress.Address_ID;
                newClient.Client_Name = clientNameTextBox.Text;

                //insert the client
                dc.insertClient(newClient.Client_Name, newClient.Billing_Address, newClient.Shipping_Address);

                //refresh
                RefreshClientData();

                clientCancelButton_Click(sender, e);
            }
        }

        private void clientAddContactButton_Click(object sender, EventArgs e)
        {
            if (_clients.Count > 0 && clientDataGridView.SelectedRows.Count > 0 && clientPK != null)
            {
                //int clientPK = (int)clientDataGridView.CurrentRow.Cells[0].Value;
                int counter = 0;
                int target = 0;
                foreach (Client client in _clients)
                {
                    if (client.Client_ID == clientPK)
                    {
                        target = counter;                                                                
                    }
                    counter++;
                }

                //go!
                PopulateContactsTab();
                orderingSystemTabControl.SelectedTab = contactsTabPage;
                contactFindClientComboBox.SelectedIndex = target;
            }
            else 
            {
                MessageBox.Show("Create and select a client first.");            
            }
        }
        #pragma warning restore

    }
}
