﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnderGuardLibrary.DataTools;
using System.Windows.Forms;

namespace Capstone
{
    /* CONTACTS TAB */
    //Author: Will Matheson
    partial class OrderingSystemForm
    {
        //pk storage from datagrid (in case the selection is moved w/o clicking)
        int contactPK;

        //our pretend "constructor"
        private void PopulateContactsTab() 
        {
            contactFindClientComboBox.Items.Clear();

            //fill in client selection box with clientID, client name
            foreach (Client client in _clients)
            {
                contactFindClientComboBox.Items.Add((string)(client.Client_ID + ". " + client.Client_Name));
            }

            _contacts = new List<Contact>();
        }

        private void contactFindClientComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            //do nothing if this is being reset for some reason
            if (!(contactFindClientComboBox.SelectedIndex >= 0))
            {
                return;
            }

            //clear contact entry
            contactCancelButton_Click(sender, e);

            //get a copy of the current Client so it's easier to work with
            Client currentClient = _clients.ElementAt(contactFindClientComboBox.SelectedIndex);

            //get contacts of the selected client
            _contacts.Clear();
            _clientContacts = dc.getClientContacts(currentClient.Client_ID).ToList();
            foreach (Client_Contact clientContact in _clientContacts)
            {
                _contacts.Add(dc.getContact(clientContact.Contact_ID).Single());
            }
            //necessary after updates to contacts
            dc.Refresh(System.Data.Linq.RefreshMode.OverwriteCurrentValues, _contacts);

            //show the appropriate contacts in the grid
            contactDataGridView.RowCount = 0;
            int row = 0;
            if (_contacts.Count > 0)
            {
                contactDataGridView.Rows.Add(_contacts.Count);

                foreach (Contact contact in _contacts)
                {
                    contactDataGridView.Rows[row].Cells[0].Value = contact.Contact_ID;
                    contactDataGridView.Rows[row].Cells[1].Value = contact.Contact_Name;
                    row++;
                }

                contactDataGridView.ClearSelection();

                contactCrudModeLabel.Text = "INSERT";
                contactInsertButton.Enabled = true;
                contactCancelButton.Enabled = true;
            }
        }

        private void contactDataGridView_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (!(contactDataGridView.RowCount > 0))
            {
                return;
            }

            //store PK
            contactPK = (int)contactDataGridView.CurrentRow.Cells[0].Value;

            Contact currentContact = dc.getContact((int)contactDataGridView.CurrentRow.Cells[0].Value).Single();
            contactNameTextBox.Text = currentContact.Contact_Name;
            contactEmailTextBox.Text = currentContact.Email;
            contactPhoneTextBox.Text = currentContact.Phone_Number;
            contactFaxTextBox.Text = currentContact.Fax_Number;
            contactNotesTextBox.Text = currentContact.Notes;

            //get contact address
            Address contactAddress = dc.getAddress(currentContact.Address_Address_ID).Single();

            //populate contact address
            contactAddressTextBox.Text = contactAddress.Civic_Address;
            contactCityTextBox.Text = contactAddress.City;
            contactProvinceComboBox.SelectedItem = contactAddress.Province;
            contactPostalCodeTextBox.Text = contactAddress.Postal_Code;

            contactInsertButton.Text = "Delete";
            contactCrudModeLabel.Text = "UPDATE";
            contactEditSaveButton.Enabled = true;

        }

        private bool contactFieldCheck()
        {
            if (contactNameTextBox.Text == string.Empty)
            {
                MessageBox.Show("Contact must have a name.");
                return false;
            }

            if (contactEmailTextBox.Text == string.Empty)
            {
                MessageBox.Show("Contact must have an e-mail address.");
                return false;
            }

            if (contactPhoneTextBox.Text == string.Empty)
            {
                MessageBox.Show("Contact must have a phone number.");
                return false;
            }

            if (contactFaxTextBox.Text == string.Empty)
            {
                MessageBox.Show("Contact must have a fax number.");
                return false;
            }

            if (contactNotesTextBox.Text == string.Empty)
            {
                MessageBox.Show("Contact must have a note.");
                return false;
            }

            if (contactAddressTextBox.Text == string.Empty)
            {
                MessageBox.Show("Contact address field Address required.");
                return false;
            }

            if (contactCityTextBox.Text == string.Empty)
            {
                MessageBox.Show("Contact address field City required.");
                return false;
            }

            if (contactPostalCodeTextBox.Text == string.Empty)
            {
                MessageBox.Show("Contact address field Postal Code required.");
                return false;
            }

            return true;        
        }

        #pragma warning disable
        private void deleteContact(object sender, EventArgs e)
        {
            contactCrudModeLabel.Text = "DELETE";

            //http://stackoverflow.com/a/3036880
            DialogResult dialogResult = MessageBox.Show("Are you sure you want to delete this contact?", "Confirm Delete", MessageBoxButtons.YesNo);
            if (dialogResult == DialogResult.Yes && contactPK != null)
            {
                //MessageBox.Show("Baleeted!");
                Client currentClient = _clients.ElementAt(contactFindClientComboBox.SelectedIndex);
                //delete the join record
                dc.deleteClientContact(currentClient.Client_ID, contactPK);
                //delete the contact
                dc.deleteContact(contactPK);
                //refresh
                contactFindClientComboBox_SelectedIndexChanged(sender, e);
            }
            else if (dialogResult == DialogResult.No)
            {
                //MessageBox.Show("Ok, you're safe.");
            }
            
        }

        private void contactInsertButton_Click(object sender, EventArgs e)
        {
            if (!(contactFindClientComboBox.SelectedIndex > -1))
            {
                MessageBox.Show("Select a client first.");
                return;
            }
            if (contactInsertButton.Text == "Delete")
            {
                deleteContact(sender, e);
                return;
            }
            if (contactFieldCheck())
            {
                //MessageBox.Show("Inserting...");
                Address newContactAddress = new Address();
                newContactAddress.Civic_Address = contactAddressTextBox.Text;
                newContactAddress.City = contactCityTextBox.Text;
                newContactAddress.Province = contactProvinceComboBox.Text;
                newContactAddress.Postal_Code = contactPostalCodeTextBox.Text;

                //insert the address
                var insertContactAddressResult = dc.insertAddress(newContactAddress.Civic_Address, newContactAddress.City, newContactAddress.Province, newContactAddress.Postal_Code, "NewContact-Primary");
                newContactAddress.Address_ID = (int)insertContactAddressResult.ElementAt(0).ID;

                Contact newContact = new Contact();
                newContact.Address_ID = newContactAddress.Address_ID;
                newContact.Address_Address_ID = newContact.Address_ID;
                newContact.Contact_Name = contactNameTextBox.Text;
                newContact.Email = contactEmailTextBox.Text;
                newContact.Fax_Number = contactFaxTextBox.Text;
                newContact.Notes = contactNotesTextBox.Text;
                newContact.Phone_Number = contactPhoneTextBox.Text;

                //insert the contact and client_contact record
                var insertContactResult = dc.insertContact(newContact.Contact_Name, newContact.Email, newContact.Phone_Number, newContact.Fax_Number, newContact.Notes, newContact.Address_ID, newContact.Address_Address_ID);
                Client_Contact joinRecord = new Client_Contact();
                joinRecord.Contact_ID = (int)insertContactResult.ElementAt(0).ID;
                Client currentClient = _clients.ElementAt(contactFindClientComboBox.SelectedIndex);
                joinRecord.Client_ID = currentClient.Client_ID;
                dc.insertClientContact(joinRecord.Client_ID, joinRecord.Contact_ID);

                //refresh the view
                contactFindClientComboBox_SelectedIndexChanged(sender, e);
            }
        }

        private void contactEditSaveButton_Click(object sender, EventArgs e)
        {
            if (contactFieldCheck() && contactPK != null)
            {
                Contact oldContact = dc.getContact(contactPK).Single();
                Address newContactAddress = new Address();
                newContactAddress.Address_ID = oldContact.Address_ID;
                newContactAddress.Civic_Address = contactAddressTextBox.Text;
                newContactAddress.City = contactCityTextBox.Text;
                newContactAddress.Province = contactProvinceComboBox.Text;
                newContactAddress.Postal_Code = contactPostalCodeTextBox.Text;

                //update the address
                dc.updateAddress(newContactAddress.Address_ID, newContactAddress.Civic_Address, newContactAddress.City, newContactAddress.Province, newContactAddress.Postal_Code, "UpdatedContact-Primary");

                Contact newContact = new Contact();
                newContact.Contact_ID = oldContact.Contact_ID;
                newContact.Address_ID = newContactAddress.Address_ID;
                newContact.Address_Address_ID = newContact.Address_ID;
                newContact.Contact_Name = contactNameTextBox.Text;
                newContact.Email = contactEmailTextBox.Text;
                newContact.Fax_Number = contactFaxTextBox.Text;
                newContact.Notes = contactNotesTextBox.Text;
                newContact.Phone_Number = contactPhoneTextBox.Text;

                //insert the contact and client_contact record
                dc.updateContact(newContact.Contact_ID, newContact.Contact_Name, newContact.Email, newContact.Phone_Number, newContact.Fax_Number, newContact.Notes, newContact.Address_ID, newContact.Address_Address_ID);
                
                //refresh the view
                contactFindClientComboBox_SelectedIndexChanged(sender, e);
            }
        }
        #pragma warning restore

        private void contactCancelButton_Click(object sender, EventArgs e)
        {
            contactNameTextBox.Clear();
            contactEmailTextBox.Clear();
            contactPhoneTextBox.Clear();
            contactFaxTextBox.Clear();
            contactNotesTextBox.Clear();
            contactAddressTextBox.Clear();
            contactCityTextBox.Clear();
            contactPostalCodeTextBox.Clear();
            contactProvinceComboBox.SelectedIndex = 0;
            contactDataGridView.ClearSelection();
            contactInsertButton.Text = "Insert";
            contactCrudModeLabel.Text = "INSERT";
            contactEditSaveButton.Enabled = false;
        }

    }
}
