﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using System.Data;
using System.Windows.Forms;

namespace Capstone
{
    /* ORDERS TAB */
    //Author: Jamal Rahhali
    partial class OrderingSystemForm
    {
        private static bool ordersTabInitialized = false;
        private DataSet dataSet;
        private SqlDataAdapter clientAdapter;
        private SqlDataAdapter orderAdapter;
        private SqlDataAdapter statusAdapter;
        private SqlDataAdapter orderProductAdapter;
        private SqlDataAdapter productAdapter;
        private BindingSource clientBindingSource;
        private BindingSource orderBindingSource;
        private BindingSource orderProductBindingSource;
        private DataRow editingRow;

        public void InitializeOrdersTab()
        {
            if (ordersTabInitialized)
                return;
            else
                ordersTabInitialized = true;

            dataSet = new DataSet();

            SqlConnection conn = CapstoneGlobals.UnderGuardConnection;
            clientAdapter = new SqlDataAdapter("SELECT * FROM Clients", conn);
            orderAdapter = new SqlDataAdapter("SELECT * FROM Orders", conn);
            SqlCommandBuilder builder = new SqlCommandBuilder(orderAdapter);

            statusAdapter = new SqlDataAdapter("SELECT * FROM Status", conn);
            orderProductAdapter = new SqlDataAdapter("SELECT Order_Product_ID, Order_Product.Product_ID, Order_ID, Qty, Product_Name, Price FROM Order_Product INNER JOIN Products ON (Order_Product.Product_ID = Products.Product_ID)", conn);
          
            orderProductAdapter.UpdateCommand = CapstoneGlobals.UnderGuardConnection.CreateCommand();
            orderProductAdapter.UpdateCommand.CommandText =
                "UPDATE Order_Product SET Product_ID = @ProductID, Qty = @Qty WHERE Order_Product_ID = @OrderProductID";
            orderProductAdapter.UpdateCommand.Parameters.Add("@ProductID", SqlDbType.Int, sizeof(Int32), "Product_ID");
            orderProductAdapter.UpdateCommand.Parameters.Add("@Qty", SqlDbType.Int, sizeof(Int32), "Qty");
            orderProductAdapter.UpdateCommand.Parameters.Add("@OrderProductID", SqlDbType.Int, sizeof(Int32), "Order_Product_ID");

            orderProductAdapter.InsertCommand = CapstoneGlobals.UnderGuardConnection.CreateCommand();
            orderProductAdapter.InsertCommand.CommandText =
                "INSERT INTO Order_Product (Product_ID, Order_ID, Qty) VALUES(@ProductID, @OrderID, @Qty)";
            orderProductAdapter.InsertCommand.Parameters.Add("@ProductID", SqlDbType.Int, sizeof(Int32), "Product_ID");
            orderProductAdapter.InsertCommand.Parameters.Add("@OrderID", SqlDbType.Int, sizeof(Int32), "Order_ID");
            orderProductAdapter.InsertCommand.Parameters.Add("@Qty", SqlDbType.Int, sizeof(Int32), "Qty");            

            productAdapter = new SqlDataAdapter("SELECT * FROM Products", conn);
            
            clientAdapter.Fill(dataSet, "Client");
            orderAdapter.Fill(dataSet, "Order");
            statusAdapter.Fill(dataSet, "Status");
            orderProductAdapter.Fill(dataSet, "OrderProduct");
            productAdapter.Fill(dataSet, "Product");

            dataSet.Relations.Add("ClientOrderRelation",
                dataSet.Tables["Client"].Columns["Client_ID"],
                dataSet.Tables["Order"].Columns["Client_ID"]);
            dataSet.Relations.Add("OrderProductRelation",
                dataSet.Tables["Order"].Columns["Order_ID"],
                dataSet.Tables["OrderProduct"].Columns["Order_ID"]);

            DataColumn[] pkColumn = new DataColumn[2];
            pkColumn[0] = dataSet.Tables["OrderProduct"].Columns["Order_ID"];
            pkColumn[1] = dataSet.Tables["OrderProduct"].Columns["Product_ID"];
            dataSet.Tables["OrderProduct"].PrimaryKey = pkColumn;

            
            pkColumn = new DataColumn[1];
            pkColumn[0] = dataSet.Tables["Order"].Columns["Order_ID"];
            dataSet.Tables["Order"].PrimaryKey = pkColumn;
            

            clientBindingSource = new BindingSource();
            clientBindingSource.DataSource = dataSet;
            clientBindingSource.DataMember = "Client";

            orderBindingSource = new BindingSource();
            orderBindingSource.DataSource = clientBindingSource;
            orderBindingSource.DataMember = "ClientOrderRelation";

            orderProductBindingSource = new BindingSource();
            orderProductBindingSource.DataSource = orderBindingSource;
            orderProductBindingSource.DataMember = "OrderProductRelation";

            orderFindClientComboBox.DataSource = clientBindingSource;
            orderFindClientComboBox.ValueMember = "Client_ID";
            orderFindClientComboBox.DisplayMember = "Client_Name";

            orderFindOrderComboBox.DataSource = orderBindingSource;
            orderFindOrderComboBox.ValueMember = "Order_ID";
            orderFindOrderComboBox.DisplayMember = "Order_ID";

            createOrderDetailDataGridView.AutoGenerateColumns = false;
            createOrderDetailDataGridView.DataSource = orderProductBindingSource;            
            createOrderDetailDataGridView.Columns[0].DataPropertyName = "Order_Product_ID";
            createOrderDetailDataGridView.Columns[1].DataPropertyName = "Product_Name";
            createOrderDetailDataGridView.Columns[2].DataPropertyName = "Price";
            createOrderDetailDataGridView.Columns[2].DefaultCellStyle.Format = "C";
            createOrderDetailDataGridView.Columns[3].DataPropertyName = "Qty";

            orderClientComboBox.DataSource = dataSet.Tables["Client"];
            orderClientComboBox.DisplayMember = "Client_Name";
            orderClientComboBox.ValueMember = "Client_ID";
            orderClientComboBox.DataBindings.Add("SelectedValue", orderBindingSource, "Client_ID");

            orderStatusComboBox.DataSource = dataSet.Tables["Status"];
            orderStatusComboBox.DisplayMember = "Current_Status";
            orderStatusComboBox.ValueMember = "Status_ID";
            orderStatusComboBox.DataBindings.Add("SelectedValue", orderBindingSource, "Status_Status_ID");

            createOrderDetailsProductComboBox.DataSource = dataSet.Tables["Product"];
            createOrderDetailsProductComboBox.DisplayMember = "Product_Name";
            createOrderDetailsProductComboBox.ValueMember = "Product_ID";
            createOrderDetailsProductComboBox.DataBindings.Add("SelectedValue", orderProductBindingSource, "Product_ID");

            createOrderDetailsQtyMaskedTextBox.DataBindings.Add("Text", orderProductBindingSource, "Qty");

            orderGroupBox.Enabled = false;
            createOrderDetailsGroupBox.Enabled = false;
            createOrderCancelButton.Enabled = false;
            orderDetailCancelButton.Enabled = false;
            orderDetailCrudModeLabel.Text = "Viewing";
            createOrderCrudModeLabel.Text = "Viewing";
            orderDetailEditSaveButton.Text = "Edit";
            createOrderEditSaveButton.Text = "Edit";
        }

        // todo: sum up totals.
        private void orderFindOrderComboBox_SelectedValueChanged(object sender, EventArgs e)
        {
            /*
            //SqlCommand command = new SqlCommand("SELECT Products.Price
            double total = 0;
            if (orderFindOrderComboBox.Items.Count > 0)
            {
                DataRow[] rows = ((DataRowView)orderFindOrderComboBox.SelectedItem).Row.GetChildRows("OrderProductRelation");
                if (rows != null)
                {
                    foreach (DataRow row in ((DataRowView)orderFindOrderComboBox.SelectedItem).Row.GetChildRows("OrderProductRelation"))
                    {
                       //int qty = (int)row["Qty"];
                       //double price = Convert.ToDouble(((double)row["Price"]).ToString());
                       //total += qty * price;
                    }
                }
            }
            orderSubtotalVarLabel.Text = String.Format("{0:C}", total);
            double tax = total * 0.07;
            orderTaxVarLabel.Text = String.Format("{0:C}", tax);
            orderTotalVarLabel.Text = String.Format("{0:C}", total + tax);
             * */
        }

        private void orderDetailEditSaveButton_Click(object sender, EventArgs e)
        {
            //if (! (orderFindOrderComboBox.Items.Count > 0))
            //    return;

            if (orderDetailCrudModeLabel.Text == "Viewing")
            {
                orderDetailEditSaveButton.Text = "Save";
                orderDetailCancelButton.Enabled = true;
                orderDetailInsertButton.Enabled = false;
                orderDetailCrudModeLabel.Text = "Editing...";
                orderFindGroupBox.Enabled = false;
                createOrderDetailDataGridView.Enabled = false;
                createOrderDetailsGroupBox.Enabled = true;

                editingRow = ((DataRowView)createOrderDetailDataGridView.CurrentRow.DataBoundItem).Row;
                editingRow.BeginEdit();
            }
            else //saving
            {
                if (orderDetailCrudModeLabel.Text == "Editing...")
                {
                    editingRow.EndEdit();
                    DataSet changedData = dataSet.GetChanges(DataRowState.Modified);
                    orderProductAdapter.Update(changedData, "OrderProduct");
                    editingRow["Product_Name"] = createOrderDetailsProductComboBox.Text;
                    dataSet.AcceptChanges();
                }
                else //"Inserting..."
                {
                    DataRow newRow = dataSet.Tables["OrderProduct"].NewRow();
                    newRow["Product_ID"] = createOrderDetailsProductComboBox.SelectedValue;
                    newRow["Order_ID"] = (int)orderFindOrderComboBox.SelectedValue;
                    //newRow["Order_ID"] = (int)((DataRowView)createOrderDetailDataGridView.CurrentRow.DataBoundItem).Row["Order_ID"];
                    newRow["Qty"] = Convert.ToInt32(createOrderDetailsQtyMaskedTextBox.Text);
                    dataSet.RejectChanges();
                    dataSet.Tables["OrderProduct"].Rows.Add(newRow);
                    orderProductAdapter.Update(dataSet, "OrderProduct");
                    dataSet.AcceptChanges();
                    orderProductAdapter.Fill(dataSet, "OrderProduct");
                }
                orderDetailEditSaveButton.Text = "Edit";
                orderDetailCancelButton.Enabled = false;
                orderDetailInsertButton.Enabled = true;
                orderDetailCrudModeLabel.Text = "Viewing";
                orderFindGroupBox.Enabled = true;
                createOrderDetailDataGridView.Enabled = true;
                createOrderDetailsGroupBox.Enabled = false;
            }
        }

        private void orderDetailCancelButton_Click(object sender, EventArgs e)
        {
            orderDetailEditSaveButton.Text = "Edit";
            orderDetailCancelButton.Enabled = false;
            createOrderCancelButton.Enabled = false;
            orderDetailInsertButton.Enabled = true;
            orderDetailCrudModeLabel.Text = "Viewing";
            orderFindGroupBox.Enabled = true;
            createOrderDetailDataGridView.Enabled = true;
            createOrderDetailsGroupBox.Enabled = false;
            orderGroupBox.Enabled = false;

            if (editingRow != null)
                editingRow.CancelEdit();
            dataSet.RejectChanges();
        }

        private void orderDetailInsertButton_Click(object sender, EventArgs e)
        {
            //if (!(orderFindOrderComboBox.Items.Count > 0))
            //    return;

            orderDetailEditSaveButton.Text = "Save";
            orderDetailCancelButton.Enabled = true;
            orderDetailInsertButton.Enabled = false;
            orderDetailCrudModeLabel.Text = "Inserting...";
            orderFindGroupBox.Enabled = false;
            createOrderDetailDataGridView.Enabled = false;
            createOrderDetailsGroupBox.Enabled = true;
            
            createOrderDetailsProductComboBox.SelectedIndex = 0;
            createOrderDetailsQtyMaskedTextBox.Text = "";
        }

        private void createOrderEditSaveButton_Click(object sender, EventArgs e)
        {
            if (createOrderCrudModeLabel.Text == "Viewing")
            {
                createOrderEditSaveButton.Text = "Save";
                createOrderCancelButton.Enabled = true;
                createOrderInsertButton.Enabled = false;
                createOrderCrudModeLabel.Text = "Editing...";
                orderFindGroupBox.Enabled = false;
                createOrderDetailDataGridView.Enabled = false;
                orderGroupBox.Enabled = true;

                editingRow = ((DataRowView)orderFindOrderComboBox.SelectedItem).Row;
                editingRow.BeginEdit();
            }
            else //saving
            {
                if (createOrderCrudModeLabel.Text == "Editing...")
                {
                    editingRow.EndEdit();
                    DataSet changedData = dataSet.GetChanges(DataRowState.Modified);
                    orderAdapter.Update(changedData, "Order");
                    dataSet.AcceptChanges();
                }
                else //"Inserting..."
                {
                    DataSet temp = new DataSet();
                    SqlDataAdapter tempAdapter = new SqlDataAdapter("SELECT * FROM Orders", CapstoneGlobals.UnderGuardConnection);
                    SqlCommandBuilder tempBuilder = new SqlCommandBuilder(tempAdapter);
                    tempAdapter.Fill(temp, "Order");

                    DataRow newRow = temp.Tables["Order"].NewRow();
                    newRow["Client_Client_ID"] = (int)orderClientComboBox.SelectedValue;
                    newRow["Client_ID"] = (int)orderClientComboBox.SelectedValue;
                    //newRow["Client_Client_ID"] = (int)orderFindClientComboBox.SelectedValue;
                    //newRow["Client_ID"] = (int)orderFindClientComboBox.SelectedValue;
                    newRow["Status_Status_ID"] = (int)orderStatusComboBox.SelectedValue;
                    newRow["Date"] = DateTime.Now;
                    dataSet.RejectChanges();
                    temp.Tables["Order"].Rows.Add(newRow);
                    orderAdapter.Update(temp, "Order");
                    dataSet.AcceptChanges();
                    orderAdapter.Fill(dataSet, "Order");
                }
                createOrderEditSaveButton.Text = "Edit";
                createOrderCancelButton.Enabled = false;
                createOrderInsertButton.Enabled = true;
                createOrderCrudModeLabel.Text = "Viewing";
                orderFindGroupBox.Enabled = true;
                createOrderDetailDataGridView.Enabled = true;
                orderGroupBox.Enabled = false;
            }
        }

        private void createOrderCancelButton_Click(object sender, EventArgs e)
        {
            createOrderEditSaveButton.Text = "Edit";
            createOrderCancelButton.Enabled = false;
            createOrderCancelButton.Enabled = false;
            createOrderInsertButton.Enabled = true;
            createOrderCrudModeLabel.Text = "Viewing";
            orderFindGroupBox.Enabled = true;
            createOrderDetailDataGridView.Enabled = true;
            createOrderDetailsGroupBox.Enabled = false;
            orderGroupBox.Enabled = false;

            if (editingRow != null)
                editingRow.CancelEdit();
            dataSet.RejectChanges();
        }

        private void createOrderInsertButton_Click(object sender, EventArgs e)
        {
            createOrderEditSaveButton.Text = "Save";
            createOrderCancelButton.Enabled = true;
            createOrderInsertButton.Enabled = false;
            createOrderCrudModeLabel.Text = "Inserting...";
            orderFindGroupBox.Enabled = false;
            createOrderDetailDataGridView.Enabled = false;
            orderGroupBox.Enabled = true;

            orderStatusComboBox.SelectedIndex = 0;
            orderFindClientComboBox.SelectedIndex = 0;
        }
    }
}