﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace CapstoneWebApplication.RegUserPages
{
    public class ShoppingCart
    {
        private List<CartItem> cartItems = new List<CartItem>();
        private Dictionary<int, int> cartQuantity = new Dictionary<int, int>();

        public void Add(CartItem item)
        {
            if (cartQuantity.ContainsKey(item.Id))
            {
                cartQuantity[item.Id]++;
            }
            else
            {
                cartItems.Add(item);
                cartQuantity.Add(item.Id, 1);
            }
        }

        public void Remove(CartItem item)
        {
            if (cartQuantity.ContainsKey(item.Id))
            {
                cartQuantity[item.Id]--;

                if (cartQuantity[item.Id] == 0)
                {
                    cartQuantity.Remove(item.Id);
                    cartItems.Remove(item);
                }
            }
        }

        public void Remove(int id)
        {
            if (cartQuantity.ContainsKey(id))
            {
                cartQuantity[id]--;

                if (cartQuantity[id] == 0)
                {
                    cartQuantity.Remove(id);

                    foreach (CartItem item in cartItems)
                    {
                        if (item.Id == id)
                        {
                            cartItems.Remove(item);
                            break;
                        }
                    }
                }
            }
        }

        public void RemoveAll(CartItem item)
        {
            cartQuantity.Remove(item.Id);
            cartItems.Remove(item);
        }

        public void RemoveAll(int id)
        {
            cartQuantity.Remove(id);

            foreach (CartItem item in cartItems)
            {
                if (item.Id == id)
                {
                    cartItems.Remove(item);
                    break;
                }
            }
        }

        public int GetQuantity(CartItem item)
        {
            return cartQuantity[item.Id];
        }

        public double GetTax()
        {
            double totalTax = 0;

            foreach (CartItem item in cartItems)
            {
                totalTax += Math.Round(item.Price * cartQuantity[item.Id] * 0.07, 2);
            }

            return totalTax;
        }

        public double GetSubTotal()
        {
            double subTotal = 0;

            foreach (CartItem item in cartItems)
            {
                subTotal += Math.Round(item.Price * cartQuantity[item.Id], 2);
            }

            return subTotal;
        }

        public double GetTotal()
        {
            return GetSubTotal() + GetTax();
        }

        public List<CartItem> GetCartItems()
        {
            return cartItems;
        }
    }
}